/* Xmsgd 
 * Copyright (C) 2001 Thomas Vander Stichele <thomas@apestaart.org>
 * 
 * Xmsg.c: standalone Xmsg program
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * version 0.6.0
 *
 * last changed : 27/06/2001
 *
 * displays a graphical message which disappears over time
 * also displays an .xpm icon in a 50x50 window
 * 
 * latest change : calculate width of window before writing text
 *
 */

#include <X11/Xlib.h>
#include <X11/xpm.h>
#include <X11/extensions/shape.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

/* project includes */

#include "config.h"
//#include "../lib/Xmsglib.h"

/* default values */

#define IMG_X 0
#define IMG_Y 0
#define IMG_W 50
#define IMG_H 50

#define MSG_X 50
#define MSG_Y 0
#define MSG_W 950
#define MSG_H 50

/* create two windows, one for the icon and one for the message
 * put them next to each other
 * display the icon in the first
 * display the message graphically in the second
 */

/* globals */

char debug = 0;

/* function prototypes */

void usage (const char *text);

int main (int argc, char *argv[])
{
  Display* display;             /* pointer to X Display structure */
  char *display_name 
       = getenv("DISPLAY");  	/* address of the X display */
  int screen_num;               /* number of screen to place the window on */
  msg_t msg;			/* save our configuration options */
  int y;

  DEBUG (("DEBUG: parsing config options\n"));

  if (!parse_config (argc, argv, &msg))
  {
    usage ("Invalid configuration options");
    return 1;
  }

  if (argc == optind)
  {
    usage ("Please supply a line of text to display !");
    return 1;
  }

  /* save text to display */
  msg.text = (char *) malloc (strlen (argv [argc - 1]) + 1);
  strcpy (msg.text, argv [argc - 1]);

  /* open connection with the X server */
  display = XOpenDisplay (display_name);
  if (display == NULL) 
  {
    fprintf (stderr, "%s: cannot connect to X server '%s'\n",
             argv[0], display_name);
    exit(1);
  }

  /* load the font */
  LoadFont (display, &msg);

  /* calculate text width */
  msg.text_width = XTextWidth (msg.font_info, msg.text,
		 	       strlen (msg.text)) + 4 /* shadows */;

  /* get the geometry of the default screen for our display */
  screen_num = DefaultScreen (display);
  msg.display_width = DisplayWidth (display, screen_num);
  msg.display_height = DisplayHeight (display, screen_num);

  /* take lowest value of either display_width - IMG_W or text_width */
  if (msg.display_width - IMG_W > msg.text_width)
    msg.message_width = msg.text_width;
  else
    msg.message_width = msg.display_width - IMG_W;
  /* calculate geometries for window */
  y = msg.line_number * MSG_H;
  
  msg.img_w = CreateWindow (display, screen_num,  0, y, IMG_W - 1, y + IMG_H - 1);
  msg.msg_w = CreateWindow (display, screen_num, 50, y, 
                        msg.message_width, y + MSG_H - 1);

  DrawText (display, msg.msg_w, msg, msg.text);
  if (msg.icon_file != NULL)
  {
    printf ("DEBUG : Drawing XPM\n");
    DrawXpm (display, msg.img_w, msg);
  }

  XFlush (display);
  usleep (msg.duration * 1.0E6);

  KillMsg (display, &msg);

  /* close the connection to the X server. */
  XCloseDisplay (display);

  return 0;
}

void 
usage (const char *text)
{
  printf ("%s\n\n", text);
  printf ("Usage info :\n\n");
  printf ("Xmsg \t[-i (icon .xpm)] [-c (X color)] [-l (line number)]\n");
  printf ("     \t[-d (duration)] [-f (X font descr)] (text to display)\n");
}
