#!/usr/bin/perl -w
# Xmsgd 
# Copyright (C) 2001 Thomas Vander Stichele <thomas@apestaart.org>
# 
# msgd.pl: a front-end server daemon for Xmsgd
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
# Version 0.6.2
#
# last changed : 26/07/2001
#
# currently has a way of configuring subsystems with different output
# types

$pixmapdir = "/usr/local/share/pixmaps/Xmsgd";

%subsys_cfg = (
  "music" => {
    icon =>     $pixmapdir."/speaker.xpm",
    color =>    "lightblue",
    duration => "3.0"
  },
  "alarm" => {
    icon =>     $pixmapdir."/redlight.xpm",
    color =>    "red",
    duration => "10.0"
  },
  "log-info" => {
    icon =>     $pixmapdir."/log.xpm",
    color =>    "white",
    duration => "3.0"
  },
  "log-warning" => {
    icon =>     $pixmapdir."/log.xpm",
    color =>    "yellow",
    duration => "5.0"
  },
  "log-problem" => {
    icon =>     $pixmapdir."/log.xpm",
    color =>    "orange",
    duration => "7.0"
  },
  "log-critical" => {
    icon =>     $pixmapdir."/redlight.xpm",
    color =>    "red",
    duration => "15.0"
  },
  "mail" => {
    icon =>     $pixmapdir."/mail.xpm",
    color =>    "green",
    duration => "3.0"
  },
);


$MAX_LINES = 4;
$MSGD_FIFO = "/var/local/msgd.fifo";
$XMSGD_FIFO = "/var/local/Xmsgd.fifo";

sub get_line ($);

sub get_line ($)
{
# wait if none is available
  $subsys = shift;
  $min_time = 0;

  for ($i = 0; $i < $MAX_LINES; ++$i)
  {
    if ($end = $line_status[$i]{end}) 
    {
      if ($end <= time) 
      { 
        # this item is out of date, clear it and return the line number
        $line_status[$i]{end} = 0.0; 
        $end = 0.0; 
        $line_status[$i]{subsys} = "";
        return $i; 
      }
      if ($min_time == 0.0) { $min_time = $end; }
      if ($end != 0 && $end < $min_time) 
      { 
        $min_time = $end; 
      } 
    }
    else 
    { 
      # this line was never set, so use it now
      $line_status[$i]{end} = 0.0; 
      return $i; 
    }
  }

  # hmm, no line available
  # sleep the shortest time necessary
  $sleep_time = $min_time - time;
#  print "sleeping for $sleep_time\n";
#  if ($sleep_time >= 1)
#  {
    sleep ($sleep_time);
#  }
  #why not try a recursive call now ? it should work, shouldn't it ?
  $i = get_line ($subsys);
  return $i;
}

# start with a fork
if (fork ())
{
  exit;
}

# check if Xmsgd is running

$command = "ps h -C Xmsgd";
$result = `$command`;
if (!$result)
{
  system ("Xmsgd");
}

open INPUT, $MSGD_FIFO or die "Cannot open fifo $MSGD_FIFO: $!\n"; 

# we want to keep running, even when the input file might be empty
# in most cases, the input file is a FIFO
# first flush what is there already, this is good for fifo's but bad for files
#while ($line = <INPUT>) {};
while (1)
{
  while ($line = <INPUT>)
  {
    ($subsys, $text) = split (/ /, $line, 2);
    chomp $text;
#    if ($text) { chomp $text; }
    $command = "";  

    if ($subsys_cfg{$subsys})
    {
      if ($icon = $subsys_cfg{$subsys}{icon}) { $command .= "-i $icon "; }
      if ($duration = $subsys_cfg{$subsys}{duration}) 
        { $command .= "-d $duration "; }
      if ($color = $subsys_cfg{$subsys}{color}) 
        { $command .= "-c $color "; }
      $line = get_line ($subsys);
      $command .= "-l $line ";
      if ($command) { system "echo $command > $XMSGD_FIFO"; }
      system "echo \"$text\" > $XMSGD_FIFO";
      
      if (!$duration) { $duration = 0.0; }
      $line_status[$line]{end} = time + $duration;
      $line_status[$line]{subsys} = $subsys;
    }
  }
  sleep 1;
}

